\name{sae}
\alias{sae}
\title{
Shrinkage Averaging Estimation
}
\description{
This function fits shrinkage estimators (Ridge, Lasso, Random Lasso) in the context of the linear model for different tuning parameters. To incorporate tuning parameter 
selection uncertainty, a weighted average of these shrinkage estimates is used rather than just choosing the estimate relating to the tuning parameter which 
minimizes cross validation. The weight choice can be based on either optimal prediction or resampling arguments.  
}
\usage{
sae(X, shrink.method = c("Lasso", "R.Lasso", "Ridge"), averaging = c("OCV", "BW_GCV"), ycol = 1, 
B = 200, Bweights = 200, nolambda = 100,kfold = 5, qv = 5, qv2 = 5, B1 = 200, B2 = 200,
standardize = TRUE, calc.variance = TRUE, print.warnings = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{
Data matrix consisting of the outcome and all potential covariates. 
}
  \item{shrink.method}{
A string, specifying which shrinkage estimator should be used. Currently, the Ridge, Lasso and Random Lasso estimator can be used.
}
  \item{averaging}{
Option to specifiy how the shrinkage averaging weights should be calculated. 
\code{OCV} calculates optimal cross validation weights which aim in improving predicitions. 
\code{BW_GCV} incorporates tuning parameter selection uncertainty by means of resampliing.
}
  \item{ycol}{
An integer, specifying the column of the outcome.
}
  \item{B}{
Number of boostrap samples to be used for variance estimation.
}
  \item{Bweights}{
Number of boostrap samples to be used for calculating the shrinkage averaging weights if  \code{BW_GCV} is selected.
}
  \item{nolambda}{
An integer specifying the number of potential tuning parameters.
}
  \item{kfold}{
An integer \code{k}, indicating the choice of \code{k}-fold cross validation to obtain weights if \code{OCV} was chosen.
}
  \item{qv}{
Number of variables to be considered in each draw of the importance measure step for the Random Lasso estimator.
}
  \item{qv2}{
Number of variables to be considered in each draw of the estimation step for the Random Lasso estimator.
}
  \item{B1}{
Number of bootstrap samples to be used in the importance measure step for the Random Lasso estimator.
}
  \item{B2}{
Number of bootstrap samples to be used in the estimation step for the Random Lasso estimator.}
  \item{standardize}{
Logical. Should covariate data be standardized?
}
  \item{calc.variance}{
Logical. Should variance be calculated?
}
  \item{print.warnings}{
Logical. Print messages and warnings?
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
Returns an object of \code{class} `sae': 
\item{coefficients}{coefficients (including standard errors) of the shrinkage averaging estimator, the `normal' shrinkage estimator and the OLS estimator.}
\item{variable.importance}{Variable importance based on shrinkage averaging weights: For each variable those weights
where the corresponding estimates are non-zero are added up. Only applicable for the \code{Lasso} estimator. Can also be visualized with \code{\link{plot.sae}}}
\item{sae.weights}{The weights which were estimated to calculate the final shrinkage averaging estimator.}
\item{sel.weights}{Selection `weights'. Indicates which tuning parameter has minimized cross validation and was used for the standard shrinkage estimator.}
\item{complexity.parameter}{The real values of the tuning parameters.}
\item{used.method}{Returns the chosen shrinkage method}
}
\references{
Schomaker, M. (2012), Shrinkage Averaging Estimation, Statistical Papers, 53:1015-1034
}
\author{
Michael Schomaker
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{plot.sae}}
}
\examples{
# Prostate data that is also used for other Lasso Examples
# (e.g. Lasso2 library, Tibshirani, 1996, JRSS B,..)

#################
# Main Examples #
#################

library(lasso2)
data(Prostate)

# Ridge averaging with optimal cross validation based weights (averaging with OCV_5)
shrink1 <- sae(Prostate,ycol=9,shrink.method="Ridge",averaging="OCV")
shrink1
  # larger CI for shrinkage AVERAGING due to incorporation of
  # tuning parameter selection uncertainty
summary(shrink1)
plot(shrink1)

shrink2 <- sae(Prostate,ycol=9,shrink.method="Ridge",averaging="BW_GCV",B=500)
summary(shrink2)
plot(shrink2)

# Lasso (averaging) estimates with optimal cross validation based weights (OCV_10)
shrink3 <- sae(Prostate,ycol=9,shrink.method="Lasso",averaging="OCV", kfold=10)
shrink3
summary(shrink3)
plot(shrink3) # plot2: 5 important variables
plot(shrink3,display.importance=FALSE)# importance measure not visualized

# now with bootstrap weights
shrink4 <- sae(Prostate,ycol=9,shrink.method="Lasso",averaging="BW_GCV")
summary(shrink4)
plot(shrink4,xaxis="realnumbers")  #...on the scale of the bounds
                                   # considerable tuning parameter selection uncertainty
#...less Bootstrap samples  (50 for s.e.,20 for the weights) and less number of candidate bounds (70)
sae(Prostate,ycol=9,shrink.method="Lasso",averaging="BW_GCV",B=50,Bweights=20,nolambda=70)

####################
# Further Examples #
####################

# Random Lasso (OCV used for the estimation process but not for calculation of selection probabilities)
# qv=3 randomly selected variables are used to estimate selection probabilities (based on B1 bootstrap_samples)
# qv2=3 randomly selected variables are used to estimate the final Lasso (averaging) Estimator (based on B2 bootstrap_samples)
# This is computerintensive and will take a few minutes!
# (please still increase B1 and B2 for better results (>=200))
shrink5 <- sae(Prostate,ycol=9,shrink.method="R.Lasso",averaging="OCV",qv=3,qv2=3,B1=50,B2=50)
shrink5
summary(shrink5)
plot(shrink5)


}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{Shrinkage}
\keyword{Model Averaging}
